﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Crop : MonoBehaviour
{
	public Hive hive;
	public Player player;
	public GameObject killEffect;
	public GameObject growthStage1;
	public GameObject growthStage2;
	public GameObject healthCrop;
	public GameObject ammoCrop;
	public GameObject groundNotWatered;
	public GameObject groundWatered;
	[Header( "Properties" )]
	public bool occupied = false;
	public bool watered = false;
	public bool ready = false;
	public int cropType;

	private int health;
	private int growthStage;
	private float tickDelta;

	void Update()
	{
		if ( occupied && growthStage < 2 )
		{
			tickDelta -= Time.deltaTime;
			if ( tickDelta <= 0f )
			{
				tickDelta = 30f;

				if ( watered )
				{
					watered = false;
					growthStage++;
					health += 15;

					if ( growthStage >= 2 )
					{
						ready = true;
					}
				}
				else
				{
					giveDamage( 10 );
				}

				groundNotWatered.SetActive( true );
				groundWatered.SetActive( false );
				updateBillboard();
			}
		}
	}

	void updateBillboard()
	{
		growthStage1.SetActive( false );
		growthStage2.SetActive( false );
		healthCrop.SetActive( false );
		ammoCrop.SetActive( false );

		if ( occupied )
		{
			if ( growthStage == 0 )
				growthStage1.SetActive( true );
			else if ( growthStage == 1 )
				growthStage2.SetActive( true );

			if ( ready )
			{
				if ( cropType == 0 )
					healthCrop.SetActive( true );
				else if ( cropType == 1 )
					ammoCrop.SetActive( true );
			}
		}
	}

	public void plant( int type )
	{
		occupied = true;
		health = 40;
		hive.crops.Add( this );
		watered = false;
		tickDelta = 30f;
		growthStage = 0;
		ready = false;
		cropType = type;
		updateBillboard();
	}

	public void water()
	{
		if ( occupied )
		{
			watered = true;
			groundNotWatered.SetActive( false );
			groundWatered.SetActive( true );
			tickDelta = 30f;
		}
	}

	public void harvest()
	{
		if ( ready )
		{
			giveDamage( health );

			if ( cropType == 0 )
			{
				player.health += 10;
			}
			else if ( cropType == 1 )
			{
				player.gunReserveAmmo += 25;
			}
		}
	}

	public float distance( Vector3 to )
	{
		Vector3 from;
		from.x = transform.position.x;
		from.y = to.y;
		from.z = transform.position.z;
		return Vector3.Distance( from, to );
	}

	public void giveDamage( int strength )
	{
		health -= strength;

		GameObject impact = Instantiate( killEffect );
		impact.SetActive( true );
		impact.transform.position = transform.position;
		Destroy( impact, 2f );

		if ( health <= 0 )
		{
			health = 0;
			occupied = false;
			ready = false;
			updateBillboard();
			groundNotWatered.SetActive( true );
			groundWatered.SetActive( false );
			hive.crops.Remove( this );
		}
	}
}
