﻿using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

public class Enemy : MonoBehaviour
{
	public Transform playerTransform;
	public GameObject killEffect;
	public UnityEngine.AI.NavMeshAgent navAgent;
	public Hive hive;
	public Player player;
	public AudioSource audioSource;
	public AudioClip[] growlSounds;
	public GameObject soundSpotPrefab;
	public AudioClip[] hitSounds;
	[Header( "Properties" )]
	public int health;
	public Vector2 attackDelayRange;
	public int attackStrength;
	public int pointsForKill;
	public int pointsForHeadshotKill;
	
	private float navDelta;
	private Crop targetCrop = null;
	private bool agro = false;
	private float cropDelta;
	private float attackDelta;
	private float growlDelta;

	void Start()
	{
		hive.enemies.Add( this );
		navAgent.enabled = true;
	}

	void OnDestroy()
	{
		hive.enemies.Remove( this );

		if ( agro )
		{
			agro = false;
			hive.agroEnemies--;
		}
	}

	void Update()
	{
		navigate();
		attack();
		noises();
	}

	void noises()
	{
		growlDelta -= Time.deltaTime;
		if ( growlDelta <= 0f )
		{
			growlDelta = Random.Range( 5, 10 );
			audioSource.clip = growlSounds[Random.Range( 0, growlSounds.Length )];
			audioSource.Play();
		}
	}

	void navigate()
	{
		Vector3 playerPos;
		playerPos.x = playerTransform.position.x;
		playerPos.y = transform.position.y;
		playerPos.z = playerTransform.position.z;
		float playerDistance = Vector3.Distance( transform.position, playerPos );

		navDelta -= Time.deltaTime;
		cropDelta -= Time.deltaTime;

		if ( targetCrop != null )
		{
			Vector3 cropPos;
			cropPos.x = targetCrop.transform.position.x;
			cropPos.y = transform.position.y;
			cropPos.z = targetCrop.transform.position.z;
			navAgent.SetDestination( cropPos );

			if ( targetCrop.occupied == false )
			{
				targetCrop = null;
			}
		}
		else
		{
			if ( playerDistance < 5f )
			{
				if ( navDelta <= 0f || navDelta > 0.4f )
				{
					navDelta = 0.4f;
					navAgent.SetDestination( playerPos );
				}
			}
			else if ( playerDistance < 15f )
			{
				if ( navDelta <= 0f || navDelta > 1f )
				{
					navDelta = 1f;
					Vector3 offset;
					offset.x = Random.Range( -2, 2 );
					offset.y = 0f;
					offset.z = Random.Range( -2, 2 );
					navAgent.SetDestination( playerPos + offset );
				}
			}
			else if ( playerDistance < 30f )
			{
				if ( navDelta <= 0f || navDelta > 5f )
				{
					navDelta = 5f;
					navAgent.SetDestination( playerPos );
				}
			}
			else
			{
				if ( navDelta <= 0f )
				{
					navDelta = 10f;
					navAgent.SetDestination( playerPos );

					if ( agro && hive.agroEnemies > 5 && Random.Range( 0, 100 ) <= 50 )
					{
						agro = false;
						hive.agroEnemies--;
					}
				}
			}

			if ( agro == false && playerDistance < 15f && hive.agroEnemies < 8 )
			{
				agro = true;
				hive.agroEnemies++;
			}

			if ( agro == false && cropDelta <= 0f && hive.agroEnemies > 5 && hive.crops.Count > 0 )
			{
				cropDelta = 4f;

				if ( Random.Range( 0, 100 ) <= 35 )
				{
					Crop[] closestCrops = hive.crops.OrderBy( h => h.distance( transform.position ) ).ToArray();
					targetCrop = closestCrops[Random.Range( 0, Mathf.Min( closestCrops.Length, 3 ) )];
				}
			}
		}
	}

	void attack()
	{
		attackDelta -= Time.deltaTime;

		if ( targetCrop != null )
		{
			Vector3 cropPos;
			cropPos.x = targetCrop.transform.position.x;
			cropPos.y = transform.position.y;
			cropPos.z = targetCrop.transform.position.z;
			float cropDistance = Vector3.Distance( transform.position, cropPos );

			if ( cropDistance < 3f && attackDelta <= 0f )
			{
				attackDelta = Random.Range( attackDelayRange.x, attackDelayRange.y );
				targetCrop.giveDamage( attackStrength );
			}
		}
		else
		{
			Vector3 playerPos;
			playerPos.x = playerTransform.position.x;
			playerPos.y = transform.position.y;
			playerPos.z = playerTransform.position.z;
			float playerDistance = Vector3.Distance( transform.position, playerPos );

			if ( playerDistance < 2.5f && attackDelta <= 0f )
			{
				attackDelta = Random.Range( attackDelayRange.x, attackDelayRange.y );
				player.giveDamage( attackStrength );
			}
		}
	}

	public void giveDamage( int strength )
	{
		health -= strength;
		navDelta = 0f;

		GameObject soundSpot = Instantiate( soundSpotPrefab );
		soundSpot.SetActive( true );
		soundSpot.transform.position = transform.position;
		SoundSpot soundSpotScript = soundSpot.GetComponent<SoundSpot>();
		soundSpotScript.audioSource.clip = hitSounds[Random.Range( 0, hitSounds.Length )];
		soundSpotScript.audioSource.Play();

		if ( targetCrop != null )
		{
			if ( Random.Range( 0, 100 ) < 15 )
			{
				targetCrop = null;
			}
		}

		if ( health <= 0 )
		{
			health = 0;
			GameObject impact = Instantiate( killEffect );
			impact.SetActive( true );
			impact.transform.position = transform.position;

			player.givePoints( strength == 85 ? pointsForHeadshotKill : pointsForKill );

			Destroy( impact, 2f );
			Destroy( gameObject );
		}
	}
}
