﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class HUD : MonoBehaviour
{
	public Text hpText;
	public Player player;
	public Image hurtImage;
	public Text infoMessageText;
	public Text equipmentNameText;
	public Text equipmentInfoText;
	public Text pointsText;
	public Text seedInfoText;
	public Text reloadText;
	public GameObject gunFlare;
	public GameObject gunModelParent;
	public GameObject gunModel;
	public GameObject wateringCanParent;
	public GameObject wateringCanModel;
	public Waves waves;
	public Text roundText;
	public Text roundTextBig;
	public Image crosshair;
	public Camera overlayCamera;
	public AudioSource roundSound;

	private float hurtDelta;
	private float infoMessageDelta;
	private float equipmentNameDelta;
	private float pointsDelta;
	private int pointsFrom;
	private int pointsTo;
	private float gunFlareDelta;
	private float gunJoltDelta;
	private float gunBobDelta;
	private Vector3 gunModelBobPos;
	private Vector3 gunModelReloadPos;
	private float wateringCanTiltDelta;
	private Vector3 wateringCanPos;
	private float roundTextBigDelta;

	void Start()
	{
		wateringCanParent.transform.position = overlayCamera.ViewportToWorldPoint( new Vector3( 0.9f, 0.05f, 2.1f ) );
	}

	void LateUpdate()
	{
		hpText.text = "HP " + player.health;
		hurtDelta -= Time.deltaTime;
		Color hurtColor = hurtImage.color;
		hurtColor.a = Mathf.SmoothStep( 0f, 0.9f, hurtDelta / 1f );
		hurtImage.color = hurtColor;
		if ( infoMessageDelta < 0f )
		{
			infoMessageText.text = "";
		}
		infoMessageDelta -= Time.deltaTime;
		equipmentNameDelta -= Time.deltaTime;
		Color equipmentColor = equipmentNameText.color;
		equipmentColor.r = Mathf.SmoothStep( 0.7f, 0f, equipmentNameDelta / 0.5f );
		equipmentNameText.color = equipmentColor;

		pointsDelta -= Time.deltaTime;
		pointsText.text = "$" + Mathf.Lerp( pointsTo, pointsFrom, pointsDelta / 0.3f ).ToString( "n0" );

		if ( player.seedCount > 0 )
		{
			if ( player.seedType == 0 )
				seedInfoText.text = "Health Seeds";
			else if ( player.seedType == 1 )
				seedInfoText.text = "Ammo Seeds";

			seedInfoText.text += " " + player.seedCount.ToString( "n0" );
		}
		else
		{
			seedInfoText.text = "";
		}

		if ( player.equipIndex == 0 )
		{
			equipmentInfoText.text = new string( 'I', player.gunMagAmmo ) + "   " + player.gunReserveAmmo.ToString( "n0" );
		}
		else if ( player.equipIndex == 1 )
		{
			equipmentInfoText.text = new string( 'I', player.wateringCanAmmo );
		}
		
		if ( gunFlare.activeInHierarchy && gunFlareDelta <= 0f )
			gunFlare.SetActive( false );
		gunFlareDelta -= Time.deltaTime;

		gunJoltDelta -= Time.deltaTime;
		Vector3 gunEuler = gunModel.transform.eulerAngles;
		gunEuler.x = Mathf.Lerp( 0f, -40f, gunJoltDelta / 0.25f );
		gunModel.transform.eulerAngles = gunEuler;

		gunModel.transform.position = Vector3.Lerp( gunModel.transform.position, gunModelParent.transform.position + gunModelBobPos + gunModelReloadPos, Time.deltaTime * 20f );

		wateringCanTiltDelta -= Time.deltaTime;
		Vector3 canEuler = wateringCanModel.transform.eulerAngles;
		canEuler.x = Mathf.SmoothStep( 0f, 50f, wateringCanTiltDelta / 0.5f );
		wateringCanModel.transform.eulerAngles = canEuler;

		wateringCanModel.transform.position = Vector3.Lerp( wateringCanModel.transform.position, wateringCanParent.transform.position + gunModelBobPos + wateringCanPos, Time.deltaTime * 20f );

		roundTextBigDelta -= Time.deltaTime;
		if ( roundTextBigDelta <= 0f )
		{
			roundTextBig.gameObject.SetActive( false );
			roundText.text = "Round " + waves.round.ToString( "n0" );
		}
	}

	public void hurt()
	{
		hurtDelta = 1f;
	}

	public void showMessage( string message, float time )
	{
		infoMessageText.text = message;
		infoMessageDelta = time;
	}

	public void setEquipmentName( string name )
	{
		equipmentNameText.text = name;
		equipmentNameDelta = 0.5f;
	}

	public void setPoints( int amount )
	{
		if ( pointsDelta <= 0f )
		{
			pointsFrom = pointsTo;
		}
		pointsTo = amount;
		pointsDelta = 0.3f;
	}

	public void showGunFlare()
	{
		Vector3 euler = gunFlare.transform.eulerAngles;
		euler.z = Random.Range( 0, 360 );
		gunFlare.transform.eulerAngles = euler;
		gunFlareDelta = 0.03f;
		gunFlare.SetActive( true );
	}

	public void joltGun()
	{
		gunJoltDelta = 0.25f;
	}

	public void bobGun( float strength )
	{
		if ( strength <= 0f )
		{
			gunBobDelta = 0f;
		}

		gunBobDelta += Time.deltaTime * strength;
		float verticalBob = Mathf.Sin( gunBobDelta * 12f ) * 0.05f;
		float horizontalBob = Mathf.Sin( gunBobDelta * 6f ) * 0.1f;
		gunModelBobPos = new Vector3( horizontalBob, verticalBob, 0f );
	}

	public void reloadGun( bool gunDown )
	{
		if ( gunDown )
		{
			gunModelReloadPos = new Vector3( 0f, -1f, 0f );
		}
		else
		{
			gunModelReloadPos = Vector3.zero;
		}
	}

	public void tiltWateringCan()
	{
		wateringCanTiltDelta = 0.5f;
	}

	public void reloadWateringCan( bool canDown )
	{
		if ( canDown )
		{
			wateringCanPos = new Vector3( 0f, -1f, 0f );
		}
		else
		{
			wateringCanPos = Vector3.zero;
		}
	}

	public void newRound()
	{
		roundTextBig.gameObject.SetActive( true );
		roundTextBigDelta = 4f;
		roundTextBig.text = "Round " + waves.round.ToString( "n0" );
		roundText.text = "";
		roundSound.Play();
	}
}
