﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class Menu : MonoBehaviour
{
	public GameObject menuPanel;
	public Slider sensitivitySlider;
	public Text sensitivityText;
	public Settings settings;
	public Image fadeImage;
	public Camera mainCamera;
	public GameObject hudPanel;
	public Text gameOverText;
	public GameObject gameOverTrackAnchor;
	public Player player;
	public Camera overlayCamera;
	public AudioSource beginSound;
	public AudioSource gameOverSound;

	private bool paused = false;
	private float beginDelta;
	private bool begin;
	private float gameOverDelta;
	private bool gameIsOver = false;
	private float watchGameOverDelta;

	void Start()
	{
		gameBegin();
	}

	void Update()
	{		
		if ( begin )
		{
			if ( Time.renderedFrameCount > 60 )
			{
				Cursor.visible = false;
				Cursor.lockState = CursorLockMode.Locked;
				Color fadeColor = fadeImage.color;
				beginDelta -= Time.unscaledDeltaTime;
				fadeColor.a = Utility.easeOut( 0f, 1f, beginDelta / 5f );
				fadeImage.color = fadeColor;

				if ( beginDelta <= 0f )
				{
					begin = false;
					fadeImage.enabled = false;
				}
			}
		}
		else
		{
			if ( Input.GetKeyUp( KeyCode.Escape ) && gameIsOver == false )
			{
				paused = !paused;
				if ( paused )
					pause();
				else
					resume();
			}

			if ( paused )
			{
				Cursor.visible = true;
				Cursor.lockState = CursorLockMode.None;

				sensitivityText.text = "Sensitivity " + sensitivitySlider.value;
				settings.mouseLookSensitivity = sensitivitySlider.value * 0.15f;
			}
			else
			{
				Cursor.visible = false;
				Cursor.lockState = CursorLockMode.Locked;
			}
		}

		if ( gameIsOver )
		{
			gameOverDelta -= Time.deltaTime;
			Color gameOverColor = gameOverText.color;
			gameOverColor.a = Mathf.SmoothStep( 1f, 0f, gameOverDelta / 10f );
			gameOverText.color = gameOverColor;
			mainCamera.transform.position = gameOverTrackAnchor.transform.position;
			mainCamera.transform.LookAt( player.transform.position );

			if ( gameOverDelta <= 0f )
			{
				fadeImage.enabled = true;
				Color fadeColor = fadeImage.color;
				fadeColor.a = Utility.easeOut( 1f, 0f, watchGameOverDelta / 10f );
				fadeImage.color = fadeColor;
				watchGameOverDelta -= Time.deltaTime;
				if ( watchGameOverDelta <= 0f )
				{
					restart();
				}
			}
		}
	}

	public void pause()
	{
		paused = true;
		Time.timeScale = 0f;
		menuPanel.SetActive( true );
	}

	public void resume()
	{
		paused = false;
		Time.timeScale = 1f;
		menuPanel.SetActive( false );
	}

	public void restart()
	{
		UnityEngine.SceneManagement.SceneManager.LoadScene( UnityEngine.SceneManagement.SceneManager.GetActiveScene().buildIndex );
	}

	public void exit()
	{
		Application.Quit();
	}

	void gameBegin()
	{
		menuPanel.SetActive( false );
		beginDelta = 5f;
		fadeImage.enabled = true;
		fadeImage.color = Color.black;
		begin = true;
		Time.timeScale = 1f;
		gameOverText.enabled = false;
		beginSound.Play();
	}

	public void gameOver()
	{
		gameOverText.enabled = true;
		gameOverDelta = 10f;
		gameIsOver = true;
		hudPanel.SetActive( false );
		watchGameOverDelta = 10f;
		overlayCamera.enabled = false;
		gameOverSound.Play();
		beginSound.Stop();
	}
}
