﻿using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

public class Player : MonoBehaviour
{
	public Camera mainCamera;
	public CharacterController characterController;
	public Settings settings;
	public GameObject impactEffect;
	public GameObject hurtImpact;
	public HUD hud;
	public GameObject waterEffect;
	public Color hurtColor;
	public Menu menu;
	public GameObject soundSpotPrefab;
	public AudioClip[] hitSounds;
	public AudioSource audioSource;
	public AudioClip gunSound;
	public AudioClip reloadSound;
	public AudioClip waterSound;
	public AudioClip buySound;
	public AudioClip plantSound;
	public AudioClip harvestSound;
	public AudioClip bulletImpactSound;
	public AudioClip refillSound;
	public AudioClip emptyGun;
	[Header( "Properties" )]
	public float cameraHeightOffset;
	public float moveVelocity;
	public int health;
	public float moveActivityVelocity;
	public int points;
	public int seedType;
	public int seedCount;
	public Gun gun;
	public int gunMagAmmo;
	public int gunReserveAmmo;
	public int wateringCanAmmo;
	public int equipIndex;

	private Vector2 cameraDelta;
	private float invincibleDelta;
	private int activity;
	private float activityDelta;
	private Crop activeCrop;	
	private Shop activeShop;
	private float blockDelta;
	private float gunFireDelta;
	private float wateringCanFireDelta;
	private float toggleDelta;
	private int targetEquipIndex;
	private WaterWell activeWell;

	void Start()
	{
		hud.setEquipmentName( gun.gunName );
		gunMagAmmo = gun.magSize;
		hud.reloadGun( false );
		hud.reloadWateringCan( true );
	}

	void Update()
	{
		if ( Time.timeScale == 0f || health <= 0 )
			return;

		invincibleDelta -= Time.deltaTime;
		blockDelta -= Time.deltaTime;

		move();
		look();
		toggle();
		shoot();
		water();
		interact();
		doActivity();
	}

	void look()
	{
		Vector2 inputAxis;
		inputAxis.x = Input.GetAxis( "Mouse X" ) * settings.mouseLookSensitivity;
		inputAxis.y = Input.GetAxis( "Mouse Y" ) * settings.mouseLookSensitivity;

		cameraDelta.x += inputAxis.x;
		cameraDelta.y = Mathf.Clamp( cameraDelta.y + inputAxis.y, -89f, 89f );

		Vector3 cameraPosition = transform.position;
		cameraPosition += new Vector3( 0f, cameraHeightOffset, 0f );
		Vector3 cameraEuler;
		cameraEuler.x = -cameraDelta.y;
		cameraEuler.y = cameraDelta.x;
		cameraEuler.z = mainCamera.transform.eulerAngles.z;

		mainCamera.transform.position = cameraPosition;
		mainCamera.transform.eulerAngles = cameraEuler;
	}

	void move()
	{
		Vector3 inputAxis;
		inputAxis.x = Input.GetAxisRaw( "Horizontal" );
		inputAxis.y = 0f;
		inputAxis.z = Input.GetAxisRaw( "Vertical" );
		if ( inputAxis.magnitude > 1f )
		{
			inputAxis.Normalize();
		}

		Vector3 cameraYawDirection = Utility.eulerToDirection( 0f, mainCamera.transform.eulerAngles.y );
		Vector3 desiredDirection = Utility.projectionTranslate( Vector3.zero, cameraYawDirection, inputAxis );
		float velocity = activity == 0 ? moveVelocity : moveActivityVelocity;
		Vector3 moveDelta = desiredDirection * velocity * Time.deltaTime;

		characterController.Move( moveDelta );
		hud.bobGun( ( velocity / moveVelocity ) * desiredDirection.magnitude );
	}

	void toggle()
	{
		if ( activity == 6 )
			return;

		toggleDelta -= Time.deltaTime;

		if ( Input.GetKeyDown( KeyCode.Q ) )
		{
			if ( equipIndex == 0 )
			{
				targetEquipIndex = 1;
				equipIndex = -1;
				toggleDelta = 0.5f;
				hud.reloadGun( true );
			}
			else if ( equipIndex == 1 )
			{
				targetEquipIndex = 0;
				equipIndex = -1;
				toggleDelta = 0.5f;
				hud.reloadWateringCan( true );
			}
			else if ( equipIndex == -1 )
			{
				toggleDelta = 0.5f - toggleDelta;

				if ( targetEquipIndex == 0 )
				{
					targetEquipIndex = 1;
					hud.reloadWateringCan( true );
				}
				else if ( targetEquipIndex == 1 )
				{
					targetEquipIndex = 0;
					hud.reloadGun( true );
				}
			}
		}

		if ( equipIndex == -1 && toggleDelta <= 0f )
		{
			equipIndex = targetEquipIndex;

			if ( equipIndex == 0 )
			{
				hud.setEquipmentName( gun.gunName );
				gunFireDelta = 0f;
				hud.reloadGun( false );
			}
			else if ( equipIndex == 1 )
			{
				hud.setEquipmentName( "Watering Can" );
				wateringCanFireDelta = 0f;
				hud.reloadText.gameObject.SetActive( false );
				hud.reloadWateringCan( false );
			}
		}
	}

	void createSoundSpot( AudioClip clip, Vector3 pos )
	{
		GameObject soundSpot = Instantiate( soundSpotPrefab );
		soundSpot.SetActive( true );
		soundSpot.transform.position = pos;
		SoundSpot soundSpotScript = soundSpot.GetComponent<SoundSpot>();
		soundSpotScript.audioSource.clip = clip;
		soundSpotScript.audioSource.Play();
	}

	void playSound( AudioClip clip )
	{
		audioSource.clip = clip;
		audioSource.Play();
	}

	void shoot()
	{
		if ( equipIndex != 0 )
			return;

		gunFireDelta -= Time.deltaTime;
		hud.reloadText.gameObject.SetActive( (float)gunMagAmmo / gun.magSize <= 0.3f && gunReserveAmmo > 0 && activity == 0 );

		if ( gunMagAmmo > 0 && gunFireDelta <= 0f && Input.GetMouseButtonDown( 0 ) )
		{
			gunMagAmmo--;
			gunFireDelta = gun.fireRate;
			Ray ray = mainCamera.ViewportPointToRay( new Vector3( 0.5f, 0.5f, 0f ) );
			createBullet( ray );
			hud.showGunFlare();
			hud.joltGun();
			createSoundSpot( gunSound, transform.position );

			if ( gunMagAmmo <= 0 && activity == 5 )
				activity = 0;
		}
		else if ( gunMagAmmo < gun.magSize && gunReserveAmmo > 0 && activity == 0 )
		{
			if ( Input.GetKeyDown( KeyCode.R ) || ( Input.GetMouseButtonDown( 0 ) && gunMagAmmo <= 0 ) )
			{
				activity = 5;
				activityDelta = gun.reloadTime;
				hud.reloadGun( true );
				playSound( reloadSound );
			}
		}

		if ( gunMagAmmo <= 0 && Input.GetMouseButtonDown( 0 ) && activity == 0 )
		{
			playSound( emptyGun );
		}
	}

	void water()
	{
		if ( equipIndex != 1 )
			return;

		wateringCanFireDelta -= Time.deltaTime;

		if ( wateringCanFireDelta <= 0f && Input.GetMouseButtonDown( 0 ) )
		{
			if ( wateringCanAmmo > 0 )
			{
				Ray ray = mainCamera.ViewportPointToRay( new Vector3( 0.5f, 0.5f, 0f ) );
				if ( createWater( ray ) )
				{
					wateringCanAmmo--;
					wateringCanFireDelta = 0.5f;
					hud.tiltWateringCan();
				}
			}
			else
			{
				hud.showMessage( "Refill watering can at a well", 2f );
			}
		}
	}

	void createBullet( Ray ray )
	{
		RaycastHit[] rayHits = Physics.RaycastAll( ray ).OrderBy( h => h.distance ).ToArray();

		if ( rayHits.Length > 0 )
		{
			RaycastHit hit = rayHits[0];
			if ( hit.collider.gameObject == gameObject && rayHits.Length > 1 )
			{
				hit = rayHits[1];
			}
			GameObject impact = Instantiate( impactEffect );
			impact.SetActive( true );
			impact.transform.position = hit.point;
			impact.transform.eulerAngles = Utility.directionToEuler( hit.normal );
			Destroy( impact, 0.5f );

			Enemy enemy = hit.collider.gameObject.GetComponentInParent<Enemy>();
			if ( enemy != null )
			{
				Hitbox hitbox = hit.collider.gameObject.GetComponent<Hitbox>();
				if ( hitbox.part == Hitbox.Part.Head )
				{
					enemy.giveDamage( 85 );
				}
				else if ( hitbox.part == Hitbox.Part.Torso )
				{
					enemy.giveDamage( 25 );
				}
				else
				{
					enemy.giveDamage( 20 );
				}
				ParticleSystem.MainModule particleModule = impact.GetComponent<ParticleSystem>().main;
				particleModule.startColor = hurtColor;
			}
			else
			{
				createSoundSpot( bulletImpactSound, hit.point );
			}
		}
	}

	bool createWater( Ray ray )
	{
		RaycastHit[] rayHits = Physics.RaycastAll( ray, 5f ).OrderBy( h => h.distance ).ToArray();

		if ( rayHits.Length > 0 )
		{
			RaycastHit hit = rayHits[0];
			if ( hit.collider.gameObject == gameObject && rayHits.Length > 1 )
			{
				hit = rayHits[1];
			}

			if ( hit.normal.y > 0f && hit.normal.y > hit.normal.x && hit.normal.y > hit.normal.z && hit.point.y < mainCamera.transform.position.y - 0.25f )
			{
				GameObject effect = Instantiate( waterEffect );
				effect.SetActive( true );
				effect.transform.position = hit.point;
				effect.transform.eulerAngles = Utility.directionToEuler( hit.normal );
				Destroy( effect, 7f );
				createSoundSpot( waterSound, hit.point );

				Crop crop = hit.collider.gameObject.GetComponent<Crop>();
				if ( crop != null )
				{
					crop.water();
				}

				return true;
			}
		}

		return false;
	}

	void interact()
	{
		if ( activity != 0 )
			return;

		Ray ray = mainCamera.ViewportPointToRay( new Vector3( 0.5f, 0.5f, 0f ) );
		RaycastHit[] rayHits = Physics.RaycastAll( ray ).OrderBy( h => h.distance ).ToArray();
		Crop crop = null;
		Shop shop = null;
		WaterWell well = null;
		Enemy enemy = null;

		if ( rayHits.Length > 0 )
		{
			RaycastHit hit = rayHits[0];
			if ( hit.collider.gameObject == gameObject && rayHits.Length > 1 )
			{
				hit = rayHits[1];
			}
			crop = hit.collider.gameObject.GetComponentInParent<Crop>();
			shop = hit.collider.gameObject.GetComponent<Shop>();
			well = hit.collider.gameObject.GetComponent<WaterWell>();
			enemy = hit.collider.gameObject.GetComponentInParent<Enemy>();
		}

		if ( shop != null && blockDelta <= 0f )
		{
			float shopDistance = Vector3.Distance( mainCamera.transform.position, shop.transform.position );
			if ( shopDistance < 4f )
			{
				if ( shop.unlocked )
				{
					if ( points >= shop.refillCost )
					{
						hud.showMessage( "Press & hold Space to refill " + shop.productName + " [Cost: " + shop.refillCost.ToString( "n0" ) + "]", 0.2f );

						if ( Input.GetKeyDown( KeyCode.Space ) )
						{
							activity = 4;
							activityDelta = 0.2f;
							activeShop = shop;
						}
					}
					else
					{
						hud.showMessage( shop.productName + " [Cost: " + shop.refillCost.ToString( "n0" ) + "]", 0.5f );
					}
				}
				else
				{
					if ( points >= shop.unlockCost )
					{
						hud.showMessage( "Press & hold Space to unlock " + shop.productName + " [Cost: " + shop.unlockCost.ToString( "n0" ) + "]", 0.2f );

						if ( Input.GetKeyDown( KeyCode.Space ) )
						{
							activity = 3;
							activityDelta = 0.7f;
							activeShop = shop;
						}
					}
					else
					{
						hud.showMessage( shop.productName + " [Cost: " + shop.unlockCost.ToString( "n0" ) + "]", 0.2f );
					}
				}
			}
		}

		if ( crop != null )
		{
			float cropDistance = Vector3.Distance( mainCamera.transform.position, crop.transform.position );
			if ( cropDistance < 4f )
			{
				if ( crop.occupied == false )
				{
					if ( seedCount <= 0 )
					{
						hud.showMessage( "You need seeds to plant here", 0.2f );
					}
					else
					{
						string seedName = "?";
						if ( seedType == 0 )
							seedName = "Health Seed";
						else if ( seedType == 1 )
							seedName = "Ammo Seed";

						hud.showMessage( "Press & hold Space to plant " + seedName, 0.2f );
						if ( Input.GetKeyDown( KeyCode.Space ) )
						{
							activity = 1;
							activityDelta = 1.5f;
							activeCrop = crop;
						}
					}
				}
				else
				{
					if ( crop.ready )
					{
						string cropName = "?";
						if ( crop.cropType == 0 )
							cropName = "Health";
						else if ( crop.cropType == 1 )
							cropName = "Ammo";

						hud.showMessage( "Press & hold Space to harvest " + cropName, 0.2f );

						if ( Input.GetKeyDown( KeyCode.Space ) )
						{
							activity = 2;
							activityDelta = 0.5f;
							activeCrop = crop;
						}
					}
				}
			}

			if ( cropDistance < 5f && crop.occupied && crop.ready == false && crop.watered == false )
			{
				hud.showMessage( "Needs watering", 0.2f );
			}
		}

		if ( well != null && blockDelta <= 0f )
		{
			float wellDistance = Vector3.Distance( mainCamera.transform.position, well.transform.position );
			if ( wellDistance < 4f )
			{
				hud.showMessage( "Press & hold Space to refill watering can", 0.2f );
				if ( Input.GetKeyDown( KeyCode.Space ) )
				{
					activity = 6;
					activityDelta = 1.5f;
					activeWell = well;
					equipIndex = -1;
					hud.reloadGun( true );
					hud.reloadWateringCan( true );
					hud.reloadText.gameObject.SetActive( false );
				}
			}
		}

		hud.crosshair.color = enemy != null ? Color.red : Color.white;
	}

	void doActivity()
	{
		if ( activity == 1 )
		{
			activityDelta -= Time.deltaTime;
			float cropDistance = Vector3.Distance( mainCamera.transform.position, activeCrop.transform.position );
			hud.showMessage( "Planting...", 0f );

			if ( Input.GetKey( KeyCode.Space ) == false || cropDistance > 4f )
			{
				activity = 0;
			}

			if ( activityDelta <= 0f )
			{
				activity = 0;
				activeCrop.plant( seedType );
				seedCount--;
				createSoundSpot( plantSound, activeCrop.transform.position );
			}
		}
		else if ( activity == 2 )
		{
			activityDelta -= Time.deltaTime;
			float cropDistance = Vector3.Distance( mainCamera.transform.position, activeCrop.transform.position );
			hud.showMessage( "Harvesting...", 0f );

			if ( Input.GetKey( KeyCode.Space ) == false || cropDistance > 4f )
			{
				activity = 0;
			}

			if ( activityDelta <= 0f )
			{
				activity = 0;
				activeCrop.harvest();
				createSoundSpot( harvestSound, activeCrop.transform.position );
			}
		}
		else if ( activity == 3 || activity == 4 )
		{
			activityDelta -= Time.deltaTime;
			float shopDistance = Vector3.Distance( mainCamera.transform.position, activeShop.transform.position );

			if ( Input.GetKey( KeyCode.Space ) == false || shopDistance > 4f )
			{
				activity = 0;
			}

			if ( activityDelta <= 0f )
			{
				activity = 0;
				hud.showMessage( "Purchased " + activeShop.giveAmount + " " + activeShop.productName, 2f );

				if ( activeShop.unlocked == false )
				{
					points -= activeShop.unlockCost;
				}
				else
				{
					points -= activeShop.refillCost;
				}

				activeShop.unlocked = true;
				if ( seedType != activeShop.productType )
				{
					seedCount = 0;
				}
				seedType = activeShop.productType;
				seedCount += activeShop.giveAmount;
				blockDelta = 3f;
				hud.setPoints( points );
				createSoundSpot( buySound, activeShop.transform.position );
			}
		}
		else if ( activity == 5 )
		{
			activityDelta -= Time.deltaTime;

			if ( ( Input.GetMouseButtonDown( 0 ) && gunMagAmmo > 0 ) || Input.GetKeyDown( KeyCode.Q ) )
			{
				activity = 0;
				if ( Input.GetKeyDown( KeyCode.Q ) == false )
					hud.reloadGun( false );
			}

			if ( activityDelta <= 0f )
			{
				activity = 0;
				gunReserveAmmo += gunMagAmmo;
				gunMagAmmo = Mathf.Min( gunReserveAmmo, gun.magSize );
				gunReserveAmmo -= gunMagAmmo;
				hud.reloadGun( false );
			}
		}
		else if ( activity == 6 )
		{
			activityDelta -= Time.deltaTime;
			float wellDistance = Vector3.Distance( mainCamera.transform.position, activeWell.transform.position );
			hud.showMessage( "Refilling...", 0f );

			if ( Input.GetKey( KeyCode.Space ) == false || wellDistance > 4f )
			{
				activity = 0;
			}

			if ( activityDelta <= 0f )
			{
				activity = 0;
				blockDelta = 3f;
				wateringCanAmmo = 10;
				equipIndex = 1;
				hud.reloadWateringCan( false );
				hud.setEquipmentName( "Watering Can" );
				wateringCanFireDelta = 0f;
				playSound( refillSound );
			}
		}
	}

	public void giveDamage( int strength )
	{
		if ( invincibleDelta <= 0f )
		{
			invincibleDelta = 0.5f;
			health -= strength;
			hud.hurt();
			createSoundSpot( hitSounds[Random.Range( 0, hitSounds.Length )], transform.position );

			GameObject impact = Instantiate( hurtImpact );
			impact.SetActive( true );
			impact.transform.position = transform.position;
			Destroy( impact, 2f );

			if ( health <= 0 )
			{
				health = 0;
				menu.gameOver();
			}
		}
	}

	public void givePoints( int amount )
	{
		points += amount;
		hud.setPoints( points );
	}
}
